from __future__ import with_statement

import os
import re
import sys
import pprint

VARS_BLOCK = re.compile('cssvars:?([a-zA-Z_.-]*)\s*{\s*([^}]*?)\s*}',
    re.DOTALL)
VARS_BLOCK_REPLACE = re.compile('(' + VARS_BLOCK.pattern + ')', re.DOTALL)
MODULE_IMPORT = re.compile('@import\s*module\([\'\"]?([a-zA-Z_.-]*)' \
    '[\'\"]?\)\s*;', re.DOTALL)
MODULE_IMPORT_REPLACE = \
    re.compile('(' + MODULE_IMPORT.pattern + ')', re.DOTALL)
CSS_COMMENT = re.compile('/\*(.*?)\*/', re.DOTALL)
VAR_NAME = '([a-zA-Z_][a-zA-z0-9.]*[a-zA-Z0-9_])'
VAR_DECL = re.compile(VAR_NAME + '\s*:\s*([^;]*?);', re.DOTALL)
WHITESPACE = re.compile('\s+', re.DOTALL)


NAMESPACES = {}  # yes, it's global
DEFAULTS = {}


class Namespace(dict):
    defaults = DEFAULTS    # shared state -- something like 'borg pattern'

    def getvar(self, attr, default=None):
        """traverses all self and the defaults to get variable value"""
        ret = self.get(attr, default)
        if not ret:
            ret = self.defaults.get(attr, default)
        return ret


def get_varsdict(txt):
    varsdict = {}
    for nsp, varset in VARS_BLOCK.findall(txt):
        isdefault = False
        if nsp:
            isdefault = nsp.lower() in ('default', 'defaults')
            NAMESPACES[nsp] = Namespace()
        for name, value in VAR_DECL.findall(varset):
            if nsp:
                if isdefault:
                    DEFAULTS[name] = value
                NAMESPACES[nsp][name] = value
            else:
                varsdict[name] = value
    varsdict.update(NAMESPACES)
    varsdict.update(DEFAULTS)
    return varsdict


def get_imports(txt, rel=''):
    ret = []
    imports = []
    for imp in MODULE_IMPORT.findall(txt):
        absimp = os.path.abspath(os.path.normpath(os.path.join(rel, imp)))
        if os.path.exists(absimp):
            ret.append(open(absimp).read())
            imports.append(imp)
    return '\n'.join(imports), '\n'.join(ret)


def replace_vars(txt, varsdict, varmatch='[%s]', strip_empty=False,
        strip_comments=False, warning_comment=''):
    varmatch_pattern = re.compile(varmatch.replace('[', '\[').replace(']',
        '\]').replace('%s', VAR_NAME))
    unsubbed = varmatch_pattern.findall(txt)
    for name in unsubbed:
        sub = varmatch % name
        if '.' in name:
            nsp, varname = name.split('.', 1)
            namespace = varsdict.get(nsp)
            if namespace:
                sub = namespace.getvar(varname)
        else:
            sub = varsdict.get(name, sub)
        if not sub:
            sub = "NOT_FOUND"
            # this branch should never be reached
            # because of 'sub = varmatch % name' right below
            # start of 'for' loop
        txt = txt.replace(varmatch % name, sub)
    output = re.sub(VARS_BLOCK_REPLACE, '', txt)
    output = re.sub(MODULE_IMPORT_REPLACE, '', output)
    if strip_comments:
        output = re.sub(CSS_COMMENT, '', output)
    if strip_empty:
        output = '\n'.join([l for l in output.splitlines() if l.strip()])
    return warning_comment + output


def process_substitutions(target, output_extension='.css', from_stdin=False,
                          verbose=False, varmatch='[%s]',
                          strip_empty_lines=False, strip_comments=False,
                          minify=False, inbuf=sys.stdin, outbuf=sys.stdout,
                          warning_comment=''):
    absfile = outname = intxt = None
    rel = ''
    if target:
        absfile = os.path.abspath(target)
        if not os.path.exists(absfile):
            print >> sys.stderr, "input file does not exist: '%s' " % absfile
            return
        outname = os.path.splitext(absfile)[0] + output_extension
        intxt = open(absfile).read()
        rel = os.path.dirname(absfile)
    elif from_stdin:
        intxt = inbuf.read()
        rel = ''
    imports, imported_text = get_imports(intxt, rel=rel)
    import_vars = get_varsdict(imported_text)
    varsdict = get_varsdict(intxt)
    varsdict.update(import_vars)
    intxt = imported_text + intxt
    if verbose:
        print >> sys.stderr, "/* found imports: \n%s\n*/" % imports
        print >> sys.stderr, "/* found cssvars: "
        pprint.pprint(varsdict, sys.stderr)
        print >> sys.stderr, "*/"
    output = replace_vars(intxt, varsdict, varmatch,
        strip_empty_lines, strip_comments, warning_comment)
    if minify:
        output = re.sub(WHITESPACE, ' ', output)
    if outname:
        with open(outname, 'w') as outfile:
            if verbose:
                print >> sys.stderr, "'%s' output to '%s'" % (absfile, outname)
            outfile.write(output)
    elif from_stdin:
        outbuf.write(output)
