import os
import mimetypes
from wsgiref.util import FileWrapper

from subcss.smoosher import process_substitutions


DEFAULT_WARNING_COMMENT = """\
/* WARNING: Do NOT edit this file directly
 * as it was generated from '.rcss' sources
 */
"""


class MiddlewareBase(object):

    def __init__(self, app, *args, **global_conf):
        self.app = app
        if args:
            self.global_conf = args[0]
        else:
            self.global_conf = global_conf
        self.isdebug = bool(self.global_conf.get('debug'))
        if not self.isdebug:
            cfg = self.global_conf['config']
            if cfg:
                try:
                    self.isdebug = cfg.get('sandbox', 'debug')
                except Exception:
                    pass

    def debug(self, *msgs):
        if self.isdebug:
            print " ".join([str(x) for x in msgs])

    @classmethod
    def construct_middleware(cls, app, global_conf=None):
        """Class method that constructs a copy of the middleware and
        returns it to the caller. Replaces the make_* functions."""
        if not global_conf:
            global_conf = {}
        return cls(app, global_conf)


class HTTPError(object):
    """HTTP Error Application Returns an error code to the user.
    """

    def __init__(self, code, response):
        self.code = code
        self.response = response

    def __call__(self, environ, start_response):
        start_response(self.code, [('content-length', str(len(self.response))),
            ('content-type', 'text/plain')])
        return [self.response]


class FileApp(object):
    """Static File Serving Application"""

    def __init__(self, filename, file_iter=FileWrapper):
        self.filename = filename
        self.content_type = mimetypes.guess_type(self.filename)[0]
        self.file_iter = file_iter

    def __call__(self, environ, start_response):
        if not os.path.exists(self.filename):
            return HTTPError("404 Not Found", "404: File %s not found."
                % self.filename)(environ, start_response)
        try:
            openfile = open(self.filename, "rb")
        except IOError:
            return HTTPError('403 Verboten',
                '403: cannot has')(environ, start_response)
        except OSError:
            return HTTPError('404 Aint Seen Dat',
                '404: not round here at least')(environ, start_response)
        except Exception, args:
            return HTTPError('500 Whooops',
                "500: %s" % args)(environ, start_response)
        start_response("200 OK", [("Content-Type", str(self.content_type))])
        return self.file_iter(openfile, 4096 * 16)


class SubCssMiddleware(MiddlewareBase):
    """Middleware for parsing and serving 'raw' css"""

    def __init__(self, *args, **kwargs):
        super(SubCssMiddleware, self).__init__(*args, **kwargs)
        self.warning_comment = self.global_conf.get('subcss.warning_comment')
        if not self.warning_comment:
            cfg = self.global_conf.get('config')
            if cfg:
                try:
                    self.warning_comment = cfg.get('subcss', 'warning_comment')
                except Exception:
                    self.warning_comment = DEFAULT_WARNING_COMMENT

    def __call__(self, environ, start_response):
        path_translated = environ['PATH_TRANSLATED']
        base, ext = os.path.splitext(path_translated)
        rcss_file = base + '.rcss'
        self.debug('## .rcss file --> "%s"' % rcss_file)
        if ext not in ('.css', '.rcss') and not os.path.exists(rcss_file):
            self.debug("## not raw css candidate...")
            return self.app(environ, start_response)
        else:
            self.debug("## attempting to render rcss file")
            try:
                target = path_translated
                newpath = base + '.css'
                if ext == '.css':
                    target = rcss_file
                process_substitutions(target,
                    strip_empty_lines=True, strip_comments=True,
                    warning_comment=self.warning_comment)
                environ['PATH_TRANSLATED'] = newpath
                self.debug('## new PATH_TRANSLATED=%s' % newpath)
                return self.app(environ, start_response)
            except IOError:
                return HTTPError('403 Verboten',
                    '403: cannot has')(environ, start_response)
            except OSError:
                return HTTPError('404 Aint Seen Dat',
                    '404: not round here at least')(environ, start_response)
            except Exception, args:
                return HTTPError('500 Whooops',
                    "500: %s" % args)(environ, start_response)
